' GizmoCollection.cs by Charles Petzold, June 2009

Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.ObjectModel
Imports System.Collections.Specialized
Imports System.ComponentModel

Namespace ChartingLib
	Public Class GizmoCollection
		Inherits ObservableCollection(Of Gizmo)
		Private gizmoWithMaximumRevenues As Gizmo
		Private maximumRevenues_Renamed As Double

		Public Event MaximumRevenuesChanged As EventHandler

		Public Property MaximumRevenues() As Double
			Set(ByVal value As Double)
				If value <> maximumRevenues_Renamed Then
					maximumRevenues_Renamed = value

					RaiseEvent MaximumRevenuesChanged(Me, EventArgs.Empty)
				End If
			End Set
			Get
				Return maximumRevenues_Renamed
			End Get
		End Property

		Protected Overrides Sub OnCollectionChanged(ByVal args As NotifyCollectionChangedEventArgs)
			MyBase.OnCollectionChanged(args)

			If args.NewItems IsNot Nothing Then
				For Each item As INotifyPropertyChanged In args.NewItems
					AddHandler item.PropertyChanged, AddressOf OnItemPropertyChanged
				Next item
			End If

			If args.OldItems IsNot Nothing Then
				For Each item As INotifyPropertyChanged In args.OldItems
					RemoveHandler item.PropertyChanged, AddressOf OnItemPropertyChanged
				Next item
			End If

			MyBase.OnCollectionChanged(args)

			CalculateMaximumRevenues()
		End Sub

		Protected Overrides Sub ClearItems()
			For Each item As INotifyPropertyChanged In Items
				RemoveHandler item.PropertyChanged, AddressOf OnItemPropertyChanged
			Next item

			MyBase.ClearItems()
			CalculateMaximumRevenues()
		End Sub

		Private Sub OnItemPropertyChanged(ByVal sender As Object, ByVal args As PropertyChangedEventArgs)
			Dim gizmo As Gizmo = TryCast(sender, Gizmo)

			If args.PropertyName = "Revenues" Then
				If gizmo Is gizmoWithMaximumRevenues Then
					If gizmo.Revenues > MaximumRevenues Then
						MaximumRevenues = gizmo.Revenues
					Else
						CalculateMaximumRevenues()
					End If
				ElseIf gizmo.Revenues > MaximumRevenues Then
						CalculateMaximumRevenues()
				End If
			End If

		End Sub

		Private Sub CalculateMaximumRevenues()
			gizmoWithMaximumRevenues = Nothing
			MaximumRevenues = 0

			For Each gizmo As Gizmo In Items
				If gizmo.Revenues >= MaximumRevenues Then
					gizmoWithMaximumRevenues = gizmo
					MaximumRevenues = gizmo.Revenues
				End If
			Next gizmo
		End Sub
	End Class
End Namespace
