' GizmoPresenter.cs by Charles Petzold, June 2009

Imports Microsoft.VisualBasic
Imports System
Imports System.ComponentModel
Imports System.Windows.Threading

Namespace ChartingLib
	Public Class GizmoPresenter
		Implements INotifyPropertyChanged
		Private rand As New Random()
		Private displayHeight_Renamed, revenuesBase_Renamed, halfRevenuesBase_Renamed As Double
		Private gizmos_Renamed As GizmoCollection

		Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

		Public Sub New()
			Gizmos = New GizmoCollection()
			AddHandler Gizmos.MaximumRevenuesChanged, AddressOf OnMaximumRevenueChanged

			For i As Integer = 0 To 29
				Dim gizmo As New Gizmo()
				gizmo.Name = ChartingLibHelper.GenerateName()
				gizmo.Revenues = 100000 * rand.NextDouble()
				Gizmos.Add(gizmo)
			Next i

			Dim tmr As New DispatcherTimer()
			AddHandler tmr.Tick, AddressOf OnTimerTick
			tmr.Interval = TimeSpan.FromSeconds(1)
			tmr.Start()
		End Sub

		Public Property Gizmos() As GizmoCollection
			Set(ByVal value As GizmoCollection)
				If value IsNot gizmos_Renamed Then
					gizmos_Renamed = value
					OnPropertyChanged("Gizmos")
				End If
			End Set
			Get
				Return gizmos_Renamed
			End Get
		End Property

		Public Property DisplayHeight() As Double
			Set(ByVal value As Double)
				If value <> displayHeight_Renamed Then
					displayHeight_Renamed = value
					OnPropertyChanged("DisplayHeight")
					SetScalingFactor()
				End If
			End Set
			Get
				Return displayHeight_Renamed
			End Get
		End Property

		Public Property RevenuesBase() As Double
			Protected Set(ByVal value As Double)
				If value <> revenuesBase_Renamed Then
					revenuesBase_Renamed = value
					OnPropertyChanged("RevenuesBase")
					HalfRevenuesBase = RevenuesBase / 2
					SetScalingFactor()
				End If
			End Set
			Get
				Return revenuesBase_Renamed
			End Get
		End Property

		Public Property HalfRevenuesBase() As Double
			Protected Set(ByVal value As Double)
				If value <> halfRevenuesBase_Renamed Then
					halfRevenuesBase_Renamed = value
					OnPropertyChanged("HalfRevenuesBase")
				End If
			End Set
			Get
				Return halfRevenuesBase_Renamed
			End Get
		End Property

		Private Sub OnMaximumRevenueChanged(ByVal sender As Object, ByVal e As EventArgs)
			RevenuesBase = ChartingLibHelper.GetRoundedMaximum(Gizmos.MaximumRevenues, 2, 5)
		End Sub

		Private Sub SetScalingFactor()
			For Each gizmo As Gizmo In Gizmos
				gizmo.ScalingFactor = DisplayHeight / RevenuesBase
			Next gizmo
		End Sub

		Protected Overridable Sub OnPropertyChanged(ByVal propertyName As String)
			RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
		End Sub

		Private Sub OnTimerTick(ByVal sender As Object, ByVal args As EventArgs)
			Dim index As Integer = rand.Next(Gizmos.Count)
			Gizmos(index).Revenues *= 0.90 + 0.2 * rand.NextDouble()
		End Sub
	End Class
End Namespace
