' PieSlice.cs by Charles Petzold, June 2009

Imports Microsoft.VisualBasic
Imports System
Imports System.Windows
Imports System.Windows.Media
Imports System.Windows.Shapes

Namespace ChartingLib
	Public Class PieSlice
		Inherits Shape
        Public Shared ReadOnly CenterProperty As DependencyProperty = EllipseGeometry.CenterProperty.AddOwner(GetType(PieSlice), New FrameworkPropertyMetadata(AddressOf OnGeometryPropertyChanged))

        Public Shared ReadOnly RadiusProperty As DependencyProperty = DependencyProperty.Register("Radius", GetType(Double), GetType(PieSlice), New FrameworkPropertyMetadata(AddressOf OnGeometryPropertyChanged))

        Public Shared ReadOnly StartAngleProperty As DependencyProperty = DependencyProperty.Register("StartAngle", GetType(Double), GetType(PieSlice), New FrameworkPropertyMetadata(AddressOf OnGeometryPropertyChanged))

        Public Shared ReadOnly SweepAngleProperty As DependencyProperty = DependencyProperty.Register("SweepAngle", GetType(Double), GetType(PieSlice), New FrameworkPropertyMetadata(AddressOf OnGeometryPropertyChanged))

		Private Shared ReadOnly CenterAngleKey As DependencyPropertyKey = DependencyProperty.RegisterReadOnly("CenterAngle", GetType(Double), GetType(PieSlice), New FrameworkPropertyMetadata())

		Public Shared ReadOnly CenterAngleProperty As DependencyProperty = CenterAngleKey.DependencyProperty

		Private pathGeometry As New PathGeometry()
		Private pathFigure As New PathFigure()
		Private lineSegment As New LineSegment()
		Private arcSegment As New ArcSegment()

		Public Sub New()
			pathFigure.IsClosed = True
			pathFigure.Segments.Add(lineSegment)
			pathFigure.Segments.Add(arcSegment)
			pathGeometry.Figures.Add(pathFigure)
		End Sub

		Public Property Center() As Point
			Set(ByVal value As Point)
				SetValue(CenterProperty, value)
			End Set
			Get
				Return CType(GetValue(CenterProperty), Point)
			End Get
		End Property

		Public Property Radius() As Double
			Set(ByVal value As Double)
				SetValue(RadiusProperty, value)
			End Set
			Get
				Return CDbl(GetValue(RadiusProperty))
			End Get
		End Property

		Public Property StartAngle() As Double
			Set(ByVal value As Double)
				SetValue(StartAngleProperty, value)
			End Set
			Get
				Return CDbl(GetValue(StartAngleProperty))
			End Get
		End Property

		Public Property SweepAngle() As Double
			Set(ByVal value As Double)
				SetValue(SweepAngleProperty, value)
			End Set
			Get
				Return CDbl(GetValue(SweepAngleProperty))
			End Get
		End Property

		Public Property CenterAngle() As Double
			Protected Set(ByVal value As Double)
				SetValue(CenterAngleKey, value)
			End Set
			Get
				Return CDbl(GetValue(CenterAngleProperty))
			End Get
		End Property

		Private Shared Sub OnGeometryPropertyChanged(ByVal obj As DependencyObject, ByVal args As DependencyPropertyChangedEventArgs)
			TryCast(obj, PieSlice).OnGeometryPropertyChanged(args)
		End Sub

		Private Sub OnGeometryPropertyChanged(ByVal args As DependencyPropertyChangedEventArgs)
			pathFigure.StartPoint = Center

			Dim angle As Double = Math.PI * StartAngle / 180
			Dim x As Double = Center.X + Radius * Math.Sin(angle)
			Dim y As Double = Center.Y - Radius * Math.Cos(angle)
			lineSegment.Point = New Point(x, y)

			angle = Math.PI * (StartAngle + SweepAngle) / 180
			x = Center.X + Radius * Math.Sin(angle)
			y = Center.Y - Radius * Math.Cos(angle)
			arcSegment.Point = New Point(x, y)
			arcSegment.Size = New Size(Radius, Radius)
			arcSegment.IsLargeArc = SweepAngle > 180
			arcSegment.SweepDirection = SweepDirection.Clockwise

			CenterAngle = StartAngle + SweepAngle / 2

			InvalidateMeasure()
		End Sub

		Protected Overrides ReadOnly Property DefiningGeometry() As System.Windows.Media.Geometry
			Get
				Return pathGeometry
			End Get
		End Property

		Protected Overrides Function MeasureOverride(ByVal constraint As Size) As Size
			Return New Size(Center.X + Radius, Center.Y + Radius)
		End Function
	End Class
End Namespace
